package envoy.server.processors;

import static envoy.data.Message.MessageStatus.*;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.logging.Logger;

import javax.persistence.EntityExistsException;

import envoy.data.GroupMessage;
import envoy.event.MessageStatusChange;
import envoy.server.data.PersistenceManager;
import envoy.server.net.ConnectionManager;
import envoy.server.net.ObjectWriteProxy;
import envoy.util.EnvoyLog;

/**
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>GroupMessageProcessor.java</strong><br>
 * Created: <strong>18.04.2020</strong><br>
 * 
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-beta
 */
public class GroupMessageProcessor implements ObjectProcessor<GroupMessage> {

	private static final ConnectionManager	connectionManager	= ConnectionManager.getInstance();
	private static final PersistenceManager	persistenceManager	= PersistenceManager.getInstance();
	private static final Logger				logger				= EnvoyLog.getLogger(GroupCreationProcessor.class);

	@Override
	public void process(GroupMessage groupMessage, long socketID, ObjectWriteProxy writeProxy) {
		groupMessage.nextStatus();

		// Update statuses to SENT / RECEIVED depending on online status
		groupMessage.getMemberStatuses().replaceAll((memberID, status) -> connectionManager.isOnline(memberID) ? RECEIVED : SENT);

		// Set status for sender to READ
		groupMessage.getMemberStatuses().replace(groupMessage.getSenderID(), READ);

		// Increment the overall status to RECEIVED if necessary
		if (Collections.min(groupMessage.getMemberStatuses().values()) == RECEIVED) {
			groupMessage.nextStatus();

			// Notify the sender of the status change
			writeProxy.write(socketID, new MessageStatusChange(groupMessage));
		}

		// Deliver the message to the recipients that are online
		writeProxy.writeToOnlineContacts(
				persistenceManager.getGroupByID(groupMessage.getRecipientID())
					.getContacts()
					.stream()
					.filter(c -> c.getID() != groupMessage.getSenderID()),
				groupMessage);

		try {
			PersistenceManager.getInstance().addMessage(new envoy.server.data.GroupMessage(groupMessage, LocalDateTime.now()));
		} catch (EntityExistsException e) {
			logger.warning("Received a groupMessage with an ID that already exists");
		}
	}

	@Override
	public Class<GroupMessage> getInputClass() { return GroupMessage.class; }
}
